<?php
/*
Plugin Name: AppExperts
Description: Integrate App Experts builder functionality with your wordpress website
Version: 1.2.5
Author: AppExperts
Author URI: https://appexperts.io/
License: GPLv2 or later
Text Domain: app-expert
*/
if (!defined('ABSPATH')) {
	exit();
}

define('APP_EXPERT_PATH',   plugin_dir_path(__FILE__));
define('APP_EXPERT_URL',  plugin_dir_url(__FILE__));
define('APP_EXPERT_PLUGIN_VERSION',  '1.2.5');  // used in assets
define('APP_EXPERT_API_NAMESPACE',  'app-expert/v1');


if (!class_exists('App_Expert')) {
	class App_Expert
	{
		public function __construct()
		{
			register_activation_hook(__FILE__, array($this, 'plugin_activated'));
            add_action( 'plugins_loaded', array( $this, 'plugin_setup' ));
            add_action( 'init',  array( $this, 'add_plugin_cpt' ));
            add_action( 'set_user_role',  array( $this, 'set_user_topics' ),10,3);

			$this->register_freemius();			
			$this->includes();			
		}		

		public function register_freemius(){

				// Create a helper function for easy SDK access.
				function app_experts_fs() {
					global $app_experts_fs;
			
					if ( ! isset( $app_experts_fs ) ) {
						// Include Freemius SDK.
						require_once dirname(__FILE__) . '/freemius/start.php';
			
						$app_experts_fs = fs_dynamic_init( array(
                            'id'                  => '8460',
                            'slug'                => 'appexperts',
                            'type'                => 'plugin',
                            'public_key'          => 'pk_45a4ed26beae8ad2f14c9618984f9',
                            'is_premium'          => false,
                            'has_addons'          => false,
                            'has_paid_plans'      => false,
                            'menu'                => array(
                                'slug'           => 'app_experts',
                                'override_exact' => true,
                                'account'        => false,
                                'contact'        => false,
                                'support'        => false,
                                'parent'         => array(
                                    'slug' => 'admin.php?page=app_experts',
                                ),
                            ),
                            // Set the SDK to work in a sandbox mode (for development & testing).
                            // IMPORTANT: MAKE SURE TO REMOVE SECRET KEY BEFORE DEPLOYMENT.
                            'secret_key'          => 'sk_sxbpc(F9V5!SU^^nER:Zmufa5N0S8',
                        ));
					}
			
					return $app_experts_fs;
				}
			
				// Init Freemius.
				app_experts_fs();
				// Signal that SDK was initiated.
				do_action( 'app_experts_fs_loaded' );
			
				function app_experts_fs_settings_url() {
					return admin_url( 'admin.php?page=app_experts' );
				}
			
				app_experts_fs()->add_filter('connect_url', 'app_experts_fs_settings_url');
				app_experts_fs()->add_filter('after_skip_url', 'app_experts_fs_settings_url');
				app_experts_fs()->add_filter('after_connect_url', 'app_experts_fs_settings_url');
				app_experts_fs()->add_filter('after_pending_connect_url', 'app_experts_fs_settings_url');
			
		}

		public function includes()
		{
			require_once APP_EXPERT_PATH . 'includes/authentication/app-expert-api-token.php';
			require_once APP_EXPERT_PATH . 'includes/authentication/app-expert-jwt.php';
			require_once APP_EXPERT_PATH . 'includes/authentication/app-expert-authentication.php';
			require_once APP_EXPERT_PATH . 'includes/authentication/app-expert-login.php';

			// Woocommerce
			require_once APP_EXPERT_PATH . 'includes/woocommerce/app-expert-woocommerce-authentication.php';
			require_once APP_EXPERT_PATH . 'includes/woocommerce/app-expert-woocommerce-checkout.php';

			
			require_once APP_EXPERT_PATH . 'vendor/autoload.php';
			add_action('rest_api_init', array($this, 'init_app_expert_endpoints'));

			// Front Pages
			require_once APP_EXPERT_PATH . 'includes/app-expert-page-manager.php';
			
			// Admin settings page
			require_once APP_EXPERT_PATH . 'includes/admin/app-expert-settings-page.php';

			// category images
			require_once APP_EXPERT_PATH . 'includes/admin/app-expert-category-images.php';
			// send notification 
			require_once APP_EXPERT_PATH . 'includes/admin/app-expert-send-notification.php';

		}

		public function init_app_expert_endpoints()
		{

			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-i-response-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-response-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-term-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-image-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-gallery-videos-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-supports-gallery-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-tag-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-category-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-author-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-cpt-term-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-tax-image-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-tax-post-types-meta-data.php';
			require_once APP_EXPERT_PATH . 'includes/controller/response-meta-data/app-expert-post-translation-meta-data.php';

			require_once APP_EXPERT_PATH . 'includes/handler/app-expert-hierarchy-handler.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-response.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-request.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-posts-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-categories-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-tags-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-pages-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-users-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-forgot-password-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-change-password-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-register-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-comments-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-types-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-taxonomies-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-cpt-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-ct-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-search-controller.php';
			require_once APP_EXPERT_PATH . 'includes/controller/app-expert-settings-controller.php';
            require_once APP_EXPERT_PATH . 'includes/controller/app-expert-firebase-token.php';
            require_once APP_EXPERT_PATH . 'includes/controller/app-expert-notification-controller.php';


			require_once APP_EXPERT_PATH . 'includes/handler/app-expert-cpt-handler.php';
			require_once APP_EXPERT_PATH . 'includes/handler/app-expert-ct-handler.php';
			
			// Woocommerce
			require_once APP_EXPERT_PATH . 'includes/woocommerce/controller/app-expert-wc-rest-products-controller.php';
            require_once APP_EXPERT_PATH . 'includes/woocommerce/controller/app-expert-wc-orders-controller.php';
            require_once APP_EXPERT_PATH . 'includes/woocommerce/controller/app-expert-wc-rest-products-attributes-controller.php';

            // WPML
            /*if (in_array('sitepress-multilingual-cms/sitepress.php', apply_filters('active_plugins', get_option('active_plugins'))) &&
                in_array('woocommerce-multilingual/wpml-woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {*/

                            
            require_once APP_EXPERT_PATH . 'includes/controller/app-expert-wpml-controller.php';
           //}
            $include_arr =array_merge(
                glob(APP_EXPERT_PATH."includes/woocommerce/store-api/*.php"),
            );

            foreach($include_arr as $file){
                require_once($file);
            }

		}

		public function plugin_activated()
		{
			$settings = get_option('app_expert_settings');

			if (!isset($settings['secret_key']) || empty($settings['secret_key'])) {

				$settings['secret_key'] = wp_hash(get_site_url() . mt_rand(0, 10000));
				update_option('app_expert_settings', $settings);
			}
			

			// Add custom page renderer page
			if (!get_page_by_path(App_Expert_Page_Manager::PAGE_RENDERER_SLUG)) {
				$post_details = array(
					'post_title'    => 'App Experts Page Renderer',
					'post_content'  => 'You are not authorized to access this page directly',
					'post_status'   => 'publish',
					'post_author'   => 1,
					'post_type' => 'page',
					'post_slug' => App_Expert_Page_Manager::PAGE_RENDERER_SLUG
				);
				wp_insert_post($post_details);
			}

			// Add custom checkout page
			if (!get_page_by_path(App_Expert_Woocommerce_Checkout::MOBILE_CHECKOUT_PAGE_SLUG)) {

				$post_details = array(
					'post_title'    => 'Mobile App Checkout Page',
					'post_content'  => '[woocommerce_checkout]',
					'post_status'   => 'publish',
					'post_author'   => 1,
					'post_type' => 'page',
					'post_slug' => App_Expert_Woocommerce_Checkout::MOBILE_CHECKOUT_PAGE_SLUG
				);
				wp_insert_post($post_details);
			}

			// Add custom checkout page
			if (!get_page_by_path(App_Expert_Woocommerce_Checkout::MOBILE_THANK_YOU_PAGE_SLUG)) {

				$post_details = array(
					'post_title'    => 'Mobile App Woocommerce Thank You',
					'post_content'  => 'You are not authorized to access this page directly',
					'post_status'   => 'publish',
					'post_author'   => 1,
					'post_type' => 'page',
					'post_slug' => App_Expert_Woocommerce_Checkout::MOBILE_THANK_YOU_PAGE_SLUG
				);
				wp_insert_post($post_details);
			}
		}

        public function plugin_setup(){
            //create manual_push_notification_table
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-db-handler.php';
            $_runner=App_Expert_DB_Handler_Handler::get_instance();
            $_runner->run();
        }
        public function add_plugin_cpt(){
            $this->create_Notification_post_type();
        }
        public function create_Notification_post_type() {
            $labels = array(
                'name' => _x('Notifications', 'Notifications',  'app-expert'),
                'singular_name' => _x(' Notification', 'Notifications',  'app-expert'),
                'menu_name' => __(' Notifications',  'app-expert'),
                'parent_item_colon' => __('Parent  Notification',  'app-expert'),
                'all_items' => __('All  Notifications',  'app-expert'),
                'view_item' => __('View  Notification',  'app-expert'),
                'add_new_item' => __('Add New  Notification',  'app-expert'),
                'add_new' => __('Add New',  'app-expert'),
                'edit_item' => __('Edit  Notification',  'app-expert'),
                'update_item' => __('Update  Notification',  'app-expert'),
                'search_items' => __('Search  Notification',  'app-expert'),
                'not_found' => __('Not Found',  'app-expert'),
                'not_found_in_trash' => __('Not found in Trash',  'app-expert'),
            );
            $args = array(
                'rewrite' => array('slug' => ' as_notifications'),
                'label' => __(' Notifications',  'app-expert'),
                'description' => __(' Notifications',  'app-expert'),
                'labels' => $labels,
                'supports' => array('title'),
                'public' => true,
                'has_archive' => true,
                'hierarchical' => false,
                'show_ui' => true,
                'show_in_menu' => false,
                'show_in_nav_menus' => false,
                'show_in_admin_bar' => false,
                'can_export' => true,
                'exclude_from_search' => true,
                'publicly_queryable' => true,
                'menu_position' => 100,
                'menu_icon' => '',
                'capability_type' => 'post',
                'capabilities' => array(
                    'create_posts' => false, // Removes support for the "Add New" function ( use 'do_not_allow' instead of false for multisite set ups )
                ),
                'map_meta_cap' => true, // Set to `false`, if users are not allowed to edit/delete existing posts

            );
            register_post_type('as_notifications', $args);
        }
        public function set_user_topics($ID, $role, $old_roles){
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-fcm-handler.php';
            $token = get_user_meta($ID,'_as_notification_tokens',true);
            if(!empty($role)) App_Expert_FCM_Handler::setTopics($role,$token);
            foreach ($old_roles as $_role ){
                App_Expert_FCM_Handler::setTopics($_role,$token,false);
            }
        }
	}
}

new App_Expert();
