<?php


if (!defined('ABSPATH')) {
	exit();
}

if (!class_exists('App_Expert_Send_Notification')) {

	class App_Expert_Send_Notification
	{
        public $_lang,$_notification_saver_handler;

        public function __construct()
        {        
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-language-handler.php';
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-notification-handler.php';
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-fcm-handler.php';
            add_action( 'user_register', array($this, 'send_user_register_notification'),999);
            add_action( 'publish_post', array($this, 'send_create_post_notification'),999);
            add_action( 'woocommerce_order_status_changed', array($this, 'send_change_order_status_notification'));
            add_action( 'comment_approved_to_unapproved', array($this, 'send_change_product_review_status_notification'),999,1);
            add_action( 'comment_unapproved_to_approved', array($this, 'send_change_product_review_status_notification'),999,1);
            add_filter( 'ae_notifications_extra_conditions', array($this,'notifications_extra_condition'), 10,1);
            //add_action( 'woocommerce_order_note_added', array($this, 'send_add_order_notes_notification'), 10, 2);
            $this->_notification_saver_handler = new App_Expert_Notification_Handler();
        }

        function send_user_register_notification($user_id)
        {
            $this->_notification_saver_handler->send_welcome_msg($user_id);
        }

        public function notifications_extra_condition($_conditions){
            $_conditions[]="(notification.type ='order_add_note'  and notification_user.id is not null )";
            return $_conditions;
        }

        function send_add_order_notes_notification($note_id, $order)
        {
            $is_customer_note = get_comment_meta($note_id, 'is_customer_note', true);
            if($is_customer_note == 1){
                $notification_options = get_option('notification_options');
                $is_option_exist = array_key_exists('order_note_added',$notification_options['woocommerce_settings']);
                if($is_option_exist){
                    $title=[];
                    $content=[];
                    $this->_lang  = App_Expert_Language_Handler::get_active_languages();
                    $order=wc_get_order($order->ID);
                    foreach ($this->_lang as $lang=>$obj){
                        App_Expert_Language_Handler::switch_lang($lang);
                        $title[$lang]   =  __('Order Updated', 'app-expert');
                        $content[$lang] =  sprintf(__('A note is added on order %s', 'app-expert'),$order->get_order_number());
                    }
                    $user_id = $order->get_user_id();
                    $_user_token = get_user_meta($user_id,'_as_notification_tokens',true);
                    $_notification=$this->_notification_saver_handler->save($title,$content,null,[],null,"order_add_note",$order->ID,$user_id);
                    if($_user_token){
                        $_lang = App_Expert_Language_Handler::get_user_locale($user_id);
                        $_notification['title'] =  $title[$_lang];
                        $_notification['content'] =  $content[$_lang];
                        $_notification['segment']=null;
                        $_notification['attachment']=null;
                        if(!is_array($_user_token))$_user_token=[$_user_token];
                        foreach ($_user_token as $token){
                            App_Expert_FCM_Handler::sendMessage($_notification['title'],$_notification['content'],$token,$_notification,'token');
                        }
                    }
                }
            }
        }

        function send_create_post_notification($post_id)
        {
            $notification_options = get_option('notification_options');
            $is_option_exist = array_key_exists('new_blog_post_added',$notification_options['general_settings']);
            $is_sent=get_post_meta($post_id,"_as_notification_sent",true);
            if($is_sent)return;
            if($is_option_exist){
                $title=[];
                $content=[];
                $this->_lang  = App_Expert_Language_Handler::get_active_languages();
                foreach ($this->_lang as $lang=>$obj){
                    App_Expert_Language_Handler::switch_lang($lang);
                    $title[$lang]   = __('New blog post', 'app-expert');
                    $content[$lang] = __('Check out our new post', 'app-expert');
                }
                $_notification = $this->_notification_saver_handler->save($title,$content,null,[],null,"post",$post_id);
                if($_notification){
                    update_post_meta($post_id,"_as_notification_sent",1);
                    foreach ($this->_lang as $lang=>$obj){
                        $_notification['title'] =  $title[$lang];
                        $_notification['content'] =  $content[$lang];
                        $_notification['segment']=null;
                        $_notification['attachment']=null;
                        App_Expert_FCM_Handler::sendMessage($_notification['title'],$_notification['content'],"'$lang' in topics ",$_notification);
                    }
                }
            }
        }

        function send_change_order_status_notification($order_id)
        {
            $notification_options = get_option('notification_options');
            $is_option_exist = array_key_exists('order_status_changed',$notification_options['woocommerce_settings']);
            if($is_option_exist){
                $title=[];
                $content=[];
                $this->_lang  = App_Expert_Language_Handler::get_active_languages();
                $order=wc_get_order($order_id);
                foreach ($this->_lang as $lang=>$obj){
                    App_Expert_Language_Handler::switch_lang($lang);
                    $title[$lang]   =  __('Order Status changed', 'app-expert');
                    $content[$lang] =  sprintf(__('Your order number %s status changed to %s', 'app-expert'),$order->get_order_number(),$order->get_status());
                }
                $user_id = $order->get_user_id();
                $_user_token = get_user_meta($user_id,'_as_notification_tokens',true);
                $_notification=$this->_notification_saver_handler->save($title,$content,null,[],null,"order",$order_id,$user_id);
                if($_user_token){
                    $_lang             = App_Expert_Language_Handler::get_user_locale($user_id);
                    $_notification['title'] =  $title[$_lang];
                    $_notification['content'] =  $content[$_lang];
                    $_notification['segment']=null;
                    $_notification['attachment']=null;
                    if(!is_array($_user_token))$_user_token=[$_user_token];
                    foreach ($_user_token as $token){
                        App_Expert_FCM_Handler::sendMessage($_notification['title'],$_notification['content'],$token,$_notification,'token');
                    }
                }
            }
        }

        function send_change_product_review_status_notification($comment_id)
        {
            $notification_options = get_option('notification_options');
            $is_approved_option_exist = array_key_exists('product_review_approved',$notification_options['woocommerce_settings']);
            $is_unapproved_option_exist = array_key_exists('product_review_rejected',$notification_options['woocommerce_settings']);
            if($is_approved_option_exist || $is_unapproved_option_exist){
                $title=[];
                $content=[];
                $comment = get_comment($comment_id);
                $comment_status = wp_get_comment_status($comment_id);
                $post_id = $comment->comment_post_ID;
                $p=get_post($post_id);
                $this->_lang  = App_Expert_Language_Handler::get_active_languages();
                if(in_array($comment_status, ['unapproved', 'approved'])){
                    if($comment_status === 'approved'){
                        foreach ($this->_lang as $lang=>$obj) {
                            App_Expert_Language_Handler::switch_lang($lang);
                            $title[$lang] = __('Review approved', 'app-expert');
                            $content[$lang] = __('Your Review got accepted, see it now on product page', 'app-expert');
                        }
                    }else{
                        foreach ($this->_lang as $lang=>$obj) {
                            App_Expert_Language_Handler::switch_lang($lang);
                            $title[$lang] = __('Review Rejected', 'app-expert');
                            $content[$lang] = sprintf(__('unfortunately. Your review on %s got rejected by admin', 'app-expert'),$p->post_title);
                        }
                    }
                    $user_email = get_comment_author_email($comment_id);
                    $user = get_user_by_email($user_email);
                    if(!$user) return;
                    $user_id = $user->ID;
                    $_notification = $this->_notification_saver_handler->save($title,$content,null,[],null,"review",$post_id,$user_id);
                    $_user_token = get_user_meta($user_id,'_as_notification_tokens',true);
                    if($_user_token){
                        $_lang             = App_Expert_Language_Handler::get_user_locale($user_id);
                        $_notification['title'] =  $title[$_lang];
                        $_notification['content'] =  $content[$_lang];
                        $_notification['segment']=null;
                        $_notification['attachment']=null;
                        if(!is_array($_user_token))$_user_token=[$_user_token];
                        foreach ($_user_token as $token){
                            App_Expert_FCM_Handler::sendMessage($_notification['title'],$_notification['content'],$token,$_notification,'token');
                        }
                    }
                }
            } 
        }
    }
}
new App_Expert_Send_Notification();
