<?php



if (!defined('ABSPATH')) {
    exit();
}


if (!class_exists('App_Expert_Settings_Page')) {

	class App_Expert_Settings_Page
	{

		public function __construct()
		{
			add_action('admin_menu', [$this, 'add_settings_page']);
			add_action('admin_init', [$this, 'register_settings']);
            add_action('admin_enqueue_scripts',  [$this, 'register_scripts']  );
            add_action('admin_head', array($this, 'add_custom_style'));
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-language-handler.php';
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-notification-handler.php';
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-fcm-handler.php';

        }


		public function add_settings_page()
		{
            add_menu_page( __( 'App Experts Settings','app-expert'), __( 'APPExperts', 'app-expert')  , 'manage_options'         , 'app_experts',null,APP_EXPERT_URL.'assets/images/logo.svg', 100);
            add_submenu_page('app_experts',  __( 'Home'                    ,'app-expert' ), __("Home"                    ,'app-expert' ),'manage_options','app_experts', [$this, 'render_home_page'],1);
            add_submenu_page('app_experts',  __( 'Settings'                ,'app-expert' ), __("Settings"                ,'app-expert' ),'manage_options','app_experts_settings', [$this, 'render_settings_page'],2);
            $server_key = get_option('server_key');
            if(!empty($server_key)) {
                add_submenu_page('app_experts', __('Manual Push Notification', 'app-expert'), __("Manual Push Notification", 'app-expert'), 'manage_options', 'manual_push_notification', [$this, 'render_manual_push_notification_page'], 3);
                add_submenu_page('app_experts', __('Notifications', 'app-expert'), __('Notifications', 'app-expert'), 'manage_options', 'edit.php?post_type=as_notifications');
            }
        }

		public function register_settings()
		{
			register_setting('aeci_options', 'aeci_options');
			add_settings_section('App Experts', __('Categories Images settings', 'app-expert'), [$this, 'AppExpertSectionText'], 'aeci-options');
			add_settings_field('ae_included_taxonomies', __('Taxonomies', 'app-expert'), [$this, 'AppExpertExcludedTaxonomies'], 'aeci-options', 'App Experts');

			register_setting('notification_options', 'notification_options');
			add_settings_section('App Experts', __('Manage automatic notification triggers', 'app-expert'), [$this, 'AppExpertNotificationSectionText'], 'notification-options');
			add_settings_field('general_notification_setings', __('General', 'app-expert'), [$this, 'generalNotificationSettings'], 'notification-options', 'App Experts');
			// check if woocommerce plugin is active or not 
			if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
				add_settings_field('woocommerce_notification_setings', __('Woocommerce', 'app-expert'), [$this, 'woocommerceNotificationSettings'], 'notification-options', 'App Experts');
			}
		}
		
		public function AppExpertExcludedTaxonomies()
		{
			$options = get_option('aeci_options');
			$disabled_taxonomies = ['nav_menu', 'link_category', 'post_format'];
			foreach (get_taxonomies() as $tax) : if (in_array($tax, $disabled_taxonomies)) continue; ?>
				<input type="checkbox" name="aeci_options[included_taxonomies][<?php echo $tax ?>]" value="<?php echo $tax ?>" <?php checked(isset($options['included_taxonomies'][$tax])); ?> /> <?php echo $tax ;?><br />
			<?php endforeach;
		}

        public function AppExpertSectionText()
		{
			echo '<p>'.__('Please select the taxonomies that you want them to get displayed with images within the mobile application pages.', 'app-expert').'</p>';
		}

		public function generalNotificationSettings()
		{
			$options = get_option('notification_options');
			$general_options = [
				[
					'title' => __('User Signup', 'app-expert'),
					'value' => 'user_signup'
				],[
					'title' =>  __('New blog post added', 'app-expert'),
					'value' => 'new_blog_post_added'
				]
			];
			foreach ($general_options as $option ):?>
				<input type="checkbox" name="notification_options[general_settings][<?php echo $option['value'] ?>]" value="<?php echo $option['value'] ?>" <?php checked(isset($options['general_settings'][$option['value']])); ?> /> <?php echo $option['title'] ;?><br />
			<?php endforeach;
		}


		public function woocommerceNotificationSettings()
		{
			$options = get_option('notification_options');
			$woocommerce_options = [
				[
					'title' => __('Order status changed', 'app-expert'),
					'value' => 'order_status_changed'
				],
//                [
//					'title' => __('Order note added', 'app-expert'),
//					'value' => 'order_note_added'
//				],
                [
					'title' => __('Product review approved', 'app-expert'),
					'value' => 'product_review_approved'
				],[
					'title' => __('Product review rejected', 'app-expert'),
					'value' => 'product_review_rejected'
				]
			];
			foreach ($woocommerce_options as $option ):?>
				<input type="checkbox" name="notification_options[woocommerce_settings][<?php echo $option['value'] ?>]" value="<?php echo $option['value'] ?>" <?php checked(isset($options['woocommerce_settings'][$option['value']])); ?> /> <?php echo $option['title'] ;?><br />
			<?php endforeach;
		}

		public function AppExpertNotificationSectionText()
		{
			echo '<p>'.__('Please select the options that you want them to use when your website push notification.', 'app-expert').'</p>';
		}

        public function render_home_page(){
            wp_enqueue_style( 'bootstrap_css' );
            wp_enqueue_style( 'app_experts_menu_pages_css' );
            wp_enqueue_script( 'app_experts_menu_pages_js' );
            include_once APP_EXPERT_PATH."templates/menu/home.php";
        }

		public function render_settings_page()
		{
            wp_enqueue_script('copy_to_clipboard' );
            wp_enqueue_style('app_experts_settings_pages_css' );
            include_once APP_EXPERT_PATH."templates/menu/settings.php";
		}

		public function render_manual_push_notification_page()
		{
            if ( ! did_action( 'wp_enqueue_media' ) ) {
                wp_enqueue_media();
            }
            wp_enqueue_style( 'bootstrap_css' );
            wp_enqueue_style( 'select2css' );
            wp_enqueue_style( 'app_experts_manual_push_css' );
            wp_enqueue_script( 'select2' );
            wp_enqueue_script( 'bootstrap_js' );
            wp_enqueue_script( 'app_experts_upload_script' );
            include_once APP_EXPERT_PATH."templates/menu/manual-push-notification.php";
		}

        public function register_scripts(){
            wp_register_style( 'bootstrap_css', APP_EXPERT_URL . 'assets/css/bootstrap.min.css', false, '1.0.0' );
            wp_register_style( 'app_experts_menu_pages_css', APP_EXPERT_URL . 'assets/css/menu-pages.css', ['bootstrap_css'], '1.0.0' );
			wp_register_script('bootstrap_js', APP_EXPERT_URL . 'assets/js/bootstrap.js', array(), '1.0.0', true);
            wp_register_style( 'app_experts_settings_pages_css', APP_EXPERT_URL . 'assets/css/settings-page.css', ['bootstrap_css'], '1.0.0' );
            wp_register_script('copy_to_clipboard', APP_EXPERT_URL . 'assets/js/copy-to-clipboard.js', array(), '1.0.0', true);
			wp_register_script('app_experts_menu_pages_js', APP_EXPERT_URL . 'assets/js/menu-pages.js', ['jquery'], '1.0.0' );
            wp_register_style( 'select2css', '//cdnjs.cloudflare.com/ajax/libs/select2/3.4.8/select2.css', false, '1.0', 'all' );
            wp_register_style( 'app_experts_manual_push_css', APP_EXPERT_URL . 'assets/css/manual-push-page.css', false, '1.0', 'all' );
            wp_register_script( 'select2', '//cdnjs.cloudflare.com/ajax/libs/select2/3.4.8/select2.js', array( 'jquery' ), '1.0', true );
            wp_register_script('app_experts_upload_script', APP_EXPERT_URL. 'assets/js/upload_custom_script.js', array( 'jquery' ,'select2'), '1.0.0' );
        }
        public function add_custom_style(){
             echo '<style>#toplevel_page_app_experts.wp-menu-image img{width: 80%;padding:5px};</style>';
             echo '<style>#toplevel_page_app_experts .wp-menu-image  img{width: 80%;padding:5px};</style>';
        }
        public function validate_manual_push(){
            $langs = App_Expert_Language_Handler::get_active_languages();
            $errors_arr = [];
            if(isset($_POST['save_send_notification'])){
                foreach ($langs as $lang=>$obj){
                    if(empty($_POST['title'][$lang])){
                        $errors_arr['title_error'] = true;
                        break;
                    }
                    if(strlen($_POST['title'][$lang]) > 50 ){
                        $errors_arr['title_length_error'] = true;
                        break;
                    }

                    if(empty($_POST['content'][$lang])){
                        $errors_arr['content_error'] = true;
                        break;
                    }

                    if(strlen($_POST['content'][$lang]) > 100 ){
                        $errors_arr['content_length_error'] = true;
                        break;
                    }
                }
                if(!empty($_POST['target'])&&filter_var($_POST['target'], FILTER_VALIDATE_URL) === FALSE ){
                    $errors_arr['target_error'] = true;
                }
            }
            return $errors_arr;
        }
        public function save_manual_push(){
            $saved=false;
            if(isset($_POST['save_send_notification'])){
                $savedSegments=$_POST['segments']??['loggedin'];
                if(isset($_POST['is_send_to_guest'])){
                    $savedSegments[]='guest';
                }
                $_notification_saver_handler = new App_Expert_Notification_Handler();
                $_notification = $_notification_saver_handler->save($_POST['title'],$_POST['content'],$_POST['target']??null,$savedSegments??[],$_POST['image']??null);
                if($_notification){
                    $segments=[];
                    if(isset($_POST['segments']))
                    {
                        foreach ($_POST['segments'] as $s){
                            $segments[]="'$s' in topics";
                        }
                    }else{
                        $segments[]="'loggedin' in topics";
                    }
                    if(isset($_POST['is_send_to_guest'])){
                        $segments[]="'guest' in topics";
                    }
                    $langs = App_Expert_Language_Handler::get_active_languages();
                    $segments=implode(" || ",$segments);
                    $title=$_POST['title'];
                    $content=$_POST['content'];
                    $_notification['segment']=null;
                    $_notification['attachment']=null;
                    if(!empty($_notification['attachment_id'])){
                        $_notification['attachment']=wp_get_attachment_url($_notification['attachment_id']);
                    }
                    foreach ($langs as $lang=>$obj){
                        $id="'$lang' in topics".(!empty($segments)?" && ($segments)":"");
                        $_notification['title'] =  $title[$lang];
                        $_notification['content'] =  $content[$lang];
                        App_Expert_FCM_Handler::sendMessage($_notification['title'],$_notification['content'],$id,$_notification,'topic',$_notification['attachment']);
                    }
                    $saved=true;
                }
            }
            return $saved;
        }
	}
	
}
new App_Expert_Settings_Page();

		

		
