<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_Login')) {
	class App_Expert_Login {
		const  LOGIN_PATH = '/login';
		const REFRESH_TOKEN_PATH = '/refresh_token';
		public const notification_token_meta = '_as_notification_tokens';
		/**
		 * @var App_Expert_JWT
		 */
		private $app_expert_jwt;

		public function __construct() {

			$this->app_expert_jwt = new App_Expert_JWT();

			add_action( 'rest_api_init', array( $this, 'register_login_route' ) );
		}

		public function register_login_route() {

			register_rest_route( APP_EXPERT_API_NAMESPACE, self::LOGIN_PATH, array(
				'methods'  => WP_REST_Server::CREATABLE,
				'callback' => array( $this, 'handle_user_login' ),
                'permission_callback' => '__return_true',
				'args' => $this->get_login_parameters()

			) );

			register_rest_route( APP_EXPERT_API_NAMESPACE, self::REFRESH_TOKEN_PATH, array(
				'methods'  => WP_REST_Server::CREATABLE,
				'callback' => array( $this, 'handle_refresh_token' ),
                'permission_callback' => '__return_true',
                'args' => $this->get_refresh_token_parameters()
			) );
		}

		/**
		 * @param WP_REST_Request $request
		 *
		 * @return array|WP_Error
		 */
		public function handle_user_login( WP_REST_Request $request ) {


			$username = $request->get_param( 'username' );
			$password = $request->get_param( 'password' );
			$push_token = $request->get_param( 'token' );

			if ( ! $this->is_username_password_present( $username, $password ) ) {
				return new WP_Error(
					'app_expert_username_password_not_present',
					__( 'Email or password parameter is not provided', 'app-expert' ),
					array(
						'status' => 400,
					)
				);
			}

			$user = wp_authenticate( $username, $password );

			if ( is_wp_error( $user ) ) {
				return new WP_Error(
					'app_expert_error_authenticating_user',
					__( 'Wrong email or password', 'app-expert' ),
					array(
						'status' => 401,
					)
				);
			}
            if(!empty($_REQUEST['wpml_language'])){
                update_user_meta($user->ID,'_as_current_lang',$_REQUEST['wpml_language']);
            }
			$jwt_token =  $this->app_expert_jwt->get_jwt_token( $user );
			if(!empty($push_token)){
				// add push token to user meta in _as_notification_tokens key
				$user_id=$user->ID;
				$tokens = get_user_meta($user_id,self::notification_token_meta,true);

				if(!is_array($tokens)) $tokens=[];
				if(!in_array($push_token,array_values($tokens))) $tokens[]=$push_token;
				update_user_meta($user_id,self::notification_token_meta,$tokens);

				// add user role to topics
				$user_roles = $user->roles;
				foreach ($user_roles as $role ){
					App_Expert_FCM_Handler::setTopics($role,$tokens);
				}
			}
			

			if ( is_a( $jwt_token, 'Exception' ) ) {
				return new WP_Error(
					$jwt_token->getMessage(),
					__( 'Something happened while getting the token', 'app-expert' ),
					array(
						'status' => 500,
					)
				);
			}
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-notification-handler.php';
            $_notification_saver_handler = new App_Expert_Notification_Handler();
            $_notification_saver_handler->send_welcome_msg($user_id);
			return $this->get_response($user , $jwt_token);

		}

		public function handle_refresh_token( WP_REST_Request $request ) {
			$refresh_token = $request->get_param( 'refresh_token' );

			if ( ! $this->is_refresh_token_present( $refresh_token ) ) {
				return new WP_Error(
					'app_expert_no_refresh_token_error',
					__( 'refresh_token parameter is not provided', 'app-expert' ),
					array(
						'status' => 400,
					)
				);
			}


			$refresh_token_decoded = $this->app_expert_jwt->decode_refresh_token( $refresh_token );

			if ( is_a( $refresh_token_decoded, 'Exception' ) ) {
				return new WP_Error(
					$refresh_token_decoded->getMessage(),
					__( 'Something happened while decoding the token', 'app-expert' ),
					array(
						'status' => 401,
					)
				);
			}

			if ( ! $this->app_expert_jwt->is_refresh_token_valid( $refresh_token ) ) {
				return new WP_Error(
					"app_expert_user_no_refresh_token_or_don't_match_current_token",
					__( "This user doesn't have a refresh token or the token doesn't match the current token.", 'app-expert' ),
					array(
						'status' => 401,
					)
				);
			}

			$user = $this->app_expert_jwt->get_user_by_refresh_token($refresh_token)[0];

			$token = $this->app_expert_jwt->get_jwt_token($user);

			return $this->get_response($user , $token);

		}

		/**
		 * @param WP_User $user
		 * @param array $token
		 *
		 * @return array|WP_Error
		 */
		private function get_response(WP_User $user , array  $token){

			$user_data = App_Expert_Response::prepare_user_data_for_response($user);

			$response = New WP_REST_Response();
            $user_data=apply_filters('ae_login_user_data_object',$user_data);

			$response->set_data(array_merge($token , $user_data));

			$response = rest_ensure_response($response);

			if (is_wp_error($response))
				return $response;

			return App_Expert_Response::get_api_single_success_response(
				$response,
				'app_expert_retieved_token',
				'retrieved Token'
			);
		}


		/**
		 * @param $username
		 * @param $password
		 *
		 * @return bool
		 */
		private function is_username_password_present( $username, $password ) {
			return isset( $username ) && ! empty( $username )
			       && isset( $password ) && ! empty( $password );
		}

		/**
		 * @param $refresh_token
		 *
		 * @return bool
		 */
		private function is_refresh_token_present( $refresh_token ) {
			return isset( $refresh_token ) && ! empty( $refresh_token );
		}

		/**
		 * @return array
		 */
		public function get_login_parameters(){
			return  array(
					'username'        => array(
						'description' => __( 'Login name for the user.' , 'app-expert' ),
						'type'        => 'string',
						'context'     => array( 'edit' ),
						'required' => true
					),
					'password'           => array(
						'description' => __( 'Login name for the user.' , 'app-expert' ),
						'type'        => 'string',
						'context'     => array(  ),
						'required'    => true,
					),
					'token'           => array(
						'description' => __( 'notification token' , 'app-expert' ),
						'type'        => 'string',
						'context'     => array(  ),
						'required'    => false,
					)

			);
		}

		/**
		 * @return array
		 */
		public function get_refresh_token_parameters(){
			return  array(
				'refresh_token'        => array(
					'description' => __( 'The Refresh Token.' , 'app-expert' ),
					'type'        => 'string',
					'context'     => array( 'edit' ),
					'required' => true
				),
			);
		}

	}
}

new App_Expert_Login();