<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_Change_Password_Controller')){

	class App_Expert_Change_Password_Controller extends  WP_REST_Controller {

		public function __construct(){
			$this->rest_base = '/users/me';
			$this->namespace = APP_EXPERT_API_NAMESPACE;
		}

		public function register_routes() {
			register_rest_route($this->namespace, '/' . $this->rest_base.'/change_password',
				array(
					array(
						'methods'             => WP_REST_Server::CREATABLE,
						'callback'            => array( $this, 'change_password' ),
						'args'                => $this->get_endpoint_args_for_item_schema(  WP_REST_Server::CREATABLE ),
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);

		}

		public function change_password(WP_REST_Request $request){
			$user_id = App_Expert_Request::get_user_id($request);

			$password = $request->get_param('new_password');

			try{
				wp_set_password($password , (int) $user_id  );
			}catch (Exception $exception){
				return new WP_Error(
					'app_expert_password_change_error',
					__( 'Something happened while setting password.' , 'app-expert' ),
					array( 'status' => 500 ) );
			}

			return array(
				'code' => 'app_expert_changed_user_password',
				'message' => __("Password changed successfully." , 'app-expert'),
				'data' => array(
					'status' => 200
				)
			);
		}

		public function get_item_schema() {
			return array(
				'$schema'    => 'http://json-schema.org/draft-04/schema#',
				'title'      => 'user',
				'type'       => 'object',
				'properties' => array(
					'current_password' => array(
						'description' => __( 'Current Password for the user (never included).'  , 'app-expert'),
						'type'        => 'string',
						'context'     => array(),
						'required'    => true,
						'arg_options' => array(
							'sanitize_callback' => array( $this, 'check_current_password' ),
						)
					),
					'new_password' => array(
						'description' => __( 'New Password for the user (never included).'  , 'app-expert'),
						'type'        => 'string',
						'context'     => array(),
						'required'    => true,
						'arg_options' => array(
							'sanitize_callback' => array( $this, 'check_new_password' ),
						)
					),
					'password_confirmation' => array(
						'description' => __( 'Confirmation Password for the user (never included).'  , 'app-expert'),
						'type'        => 'string',
						'context'     => array(),
						'required'    => true,
						'arg_options' => array(
							'sanitize_callback' => array( $this, 'check_confirmation_password' ),
						)
					),
				)
			);

		}

		/**
		 * @param mixed           $value   The password submitted in the request.
		 * @param WP_REST_Request $request Full details about the request.
		 * @param string          $param   The parameter name.
		 * @return WP_Error|string The sanitized password, if valid, otherwise an error.
		 */
		public function check_current_password($value, $request, $param){
			$current_password = (string) $value;

			if (is_wp_error($current_password))
				return $current_password;

			$user_id = App_Expert_Request::get_user_id($request);

			if (!isset($user_id) || empty($user_id)){
				return new WP_Error(
					'app_expert_no_user_id',
					__( 'No User ID was found.' , 'app-expert' ),
					array( 'status' => 401 ) );
			}

			$user = get_user_by('ID' , $user_id);

			if (is_wp_error($user))
				return $user;



			if (!wp_check_password($current_password , $user->user_pass , $user_id)){
				return new WP_Error(
					'app_expert_current_password_mismatch',
					__( 'Current password is invalid.' , 'app-expert' ),
					array( 'status' => 400 ) );
			}

			return $current_password;

		}

		/**
		 * @param mixed           $value   The password submitted in the request.
		 * @param WP_REST_Request $request Full details about the request.
		 * @param string          $param   The parameter name.
		 * @return WP_Error|string The sanitized password, if valid, otherwise an error.
		 */
		public function check_new_password($value, $request, $param){
			$new_password = (string) $value;

			$new_password = App_Expert_Register_Controller::general_password_check($new_password);


			if ( empty(trim($value)) ){
				$new_password = new WP_Error(
					'app_expert_new_password_not_present',
					__( 'New password parameter is not provided', 'app-expert' ),
					array(
						'status' => 400,
					)
				);
			}



			if (is_wp_error($new_password))
				return $new_password;



			return  $new_password;
		}

		/**
		 * @param mixed           $value   The password submitted in the request.
		 * @param WP_REST_Request $request Full details about the request.
		 * @param string          $param   The parameter name.
		 * @return WP_Error|string The sanitized password, if valid, otherwise an error.
		 */
		public function check_confirmation_password($value, $request, $param){
			$confirmation_password = (string) $value;

			$password = $request->get_params()['new_password'];

			$confirmation_password = App_Expert_Register_Controller::general_password_check($confirmation_password);

			if (is_wp_error($confirmation_password))
				return $confirmation_password;

			if ( !isset($password) || empty($password) || $password != $confirmation_password ) {
				return new WP_Error( 'rest_user_mismatch_password', __( 'Both passwords must match' , 'app-expert' ), array( 'status' => 400 ) );
			}

			return $confirmation_password;
		}


	

	}


}

$app_expert_change_password_controller = new App_Expert_Change_Password_Controller();
$app_expert_change_password_controller->register_routes();