<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_Notification_Token')) {
	class App_Expert_Notification_Token extends  WP_REST_Users_Controller{
		const  REGISTER_TOKEN_PATH = '/add_token';
		const UNREGISTER_TOKEN_PATH = '/remove_token';
        public const notification_token_meta = '_as_notification_tokens';


		public function register_firebase_token_route() {
			register_rest_route( APP_EXPERT_API_NAMESPACE, self::REGISTER_TOKEN_PATH, array(
				'methods'  => WP_REST_Server::CREATABLE,
				'callback' => array( $this, 'handel_register_token' ),
				'args' => $this->get_register_token_parameters(),
                'permission_callback' => '__return_true',

			) );

			register_rest_route( APP_EXPERT_API_NAMESPACE, self::UNREGISTER_TOKEN_PATH, array(
				'methods'  => WP_REST_Server::CREATABLE,
				'callback' => array( $this, 'handel_unregister_token' ),
				'args' => $this->get_register_token_parameters(),
                'permission_callback' => '__return_true',
            ) );
		}
        public function get_register_token_parameters(){
		    return [
                'token'        => array(
                    'description' => __( 'notification token' , 'app-expert' ),
                    'type'        => 'string',
                    'context'     => array( 'edit' ),
                    'required' => true
                ),
            ];
        }
        public function handel_register_token( WP_REST_Request $request ){
            $token = $request->get_param( 'token' );

            if(!$token){
                return new WP_Error(
                    'app_expert_no_token_error',
                    __( 'token parameter is not provided', 'app-expert' ),
                    array(
                        'status' => 400,
                    )
                );
            }
            $user_id = get_current_user_id();

            $tokens = get_user_meta($user_id,self::notification_token_meta,true);

            if(!is_array($tokens)) $tokens=[];
            if(!in_array($token,array_values($tokens))) $tokens[]=$token;
            update_user_meta($user_id,self::notification_token_meta,$tokens);

            // add user role to topics
            $user_meta = get_userdata($user_id);
            $user_roles = $user_meta->roles;
            foreach ($user_roles as $role ){
                App_Expert_FCM_Handler::setTopics($role,$tokens);
            }

            return $this->get_response("token added");
        }
        public function handel_unregister_token( WP_REST_Request $request ){
            $token = $request->get_param( 'token' );
            if(!$token){
                return new WP_Error(
                    'app_expert_no_token_error',
                    __( 'token parameter is not provided', 'app-expert' ),
                    array(
                        'status' => 400,
                    )
                );
            }
            $user_id=get_current_user_id();
            $tokens = get_user_meta($user_id,self::notification_token_meta,true);

            if(!empty($tokens)){
                if (($key = array_search($token, $tokens)) !== false) {
                    unset($tokens[$key]);
                }
                update_user_meta($user_id,self::notification_token_meta,$tokens);

                // remove user roles from topics
                $user_meta = get_userdata($user_id);
                $user_roles = $user_meta->roles;
                foreach ($user_roles as $role ){
                    App_Expert_FCM_Handler::setTopics($role,$tokens,false);
                }
            }
            

            return $this->get_response("token removed");
        }
        /**
         * @param WP_User $user
         * @param array $token
         *
         * @return array|WP_Error
         */
        private function get_response($msg){

            $response = New WP_REST_Response();

            $response->set_data(['message'=>__($msg, 'app-expert' )]);

            $response = rest_ensure_response($response);

            if (is_wp_error($response))
                return $response;

            return App_Expert_Response::get_api_single_success_response(
                $response,
                'app_expert_updated_notification_tokens',
                'Tokens updated'
            );
        }


    }
    $_ins=  new App_Expert_Notification_Token();
	$_ins->register_firebase_token_route();
}
