<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_Forgot_Password_Controller')){

	class App_Expert_Forgot_Password_Controller extends  WP_REST_Controller {

		public function __construct(){
			$this->rest_base = '/users';
			$this->namespace = APP_EXPERT_API_NAMESPACE;
		}

		public function register_routes() {
			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base.'/forgot_password',
				array(
					array(
						'methods'             => WP_REST_Server::CREATABLE,
						'callback'            => array( $this, 'forgot_password' ),
						'args'                => $this->get_endpoint_args_for_item_schema(  WP_REST_Server::CREATABLE ),
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);

		}

		public function forgot_password(WP_REST_Request $request){
			$email = $request->get_params()['email'];
			$user = get_user_by_email($email);

			if (is_wp_error($user))
				return $user;

			$user_login = $user->user_login;

			$key  = get_password_reset_key( $user);

			if (is_wp_error($key))
				return $key;

			if ( is_multisite() ) {
				$site_name = get_network()->site_name;
			} else {
				$site_name = wp_specialchars_decode( get_option( 'blogname' ), ENT_QUOTES );
			}

			$message = __( 'Someone has requested a password reset for the following account:' , 'app-expert' ) . "\r\n\r\n";
			$message .= sprintf( __( 'Site Name: %s' ,  'app-expert' ), $site_name ) . "\r\n\r\n";
			$message .= sprintf( __( 'Username: %s' ,  'app-expert' ), $user_login ) . "\r\n\r\n";
			$message .= __( 'If this was a mistake, just ignore this email and nothing will happen.' ,  'app-expert' ) . "\r\n\r\n";
			$message .= __( 'To reset your password, visit the following address:' ,  'app-expert' ) . "\r\n\r\n";
			$message .= '<' . network_site_url( "wp-login.php?action=rp&key=$key&login=" . rawurlencode( $user_login ), 'login' ) . ">\r\n";

			$title = sprintf( __( '[%s] Password Reset' , 'app-expert' ), $site_name );

			if ( $message && ! wp_mail( $email, wp_specialchars_decode( $title ), $message ) ) {
				return new WP_Error(
					"app_expert_password_reset_send_error",
					sprintf(
					/* translators: %s: Documentation URL. */
						__( '<strong>ERROR</strong>: The email could not be sent. Your site may not be correctly configured to send emails. <a href="%s">Get support for resetting your password</a>.', 'app-expert' ),
						esc_url( __( 'https://wordpress.org/support/article/resetting-your-password/' ) )
					),
					array(
						'status' => 502,
					)
				);
			}

			return array(
				'code' => 'app_expert_updated_user',
				'message' => __("An Email has been sent. Please check your email for the recovery email to reset your password. If you don't get an email check your spam folder or try again." , 'app-expert'),
				'data' => array(
					'status' => 200
				)
			);
		}

		public function get_item_schema() {
			 return array(
				'$schema'    => 'http://json-schema.org/draft-04/schema#',
				'title'      => 'user',
				'type'       => 'object',
				'properties' => array(
					'email'           => array(
						'description' => __( 'Forgot Password User Email' , 'app-expert' ),
						'type'        => 'string',
						'format'      => 'email',
						'context'     => array( 'edit' ),
						'arg_options' => array(
							'sanitize_callback' => array( $this, 'check_email' ),
						),
						'required'    => true,
					),
				)

			);

		}

		/**
		 * @param mixed           $value   The password submitted in the request.
		 * @param WP_REST_Request $request Full details about the request.
		 * @param string          $param   The parameter name.
		 * @return WP_Error|string The sanitized password, if valid, otherwise an error.
		 */
		public function check_email($value, $request, $param){
			$email = (string) $value;

			$user_email_sanitized = sanitize_email( $email );

			if ( ! is_email( $user_email_sanitized ) ) {
				return new WP_Error(
					"app_expert_invalid_email",
					__( "Please provide a valid email address.", 'app-expert' ),
					array(
						'status' => 400,
					)
				);
			}

			if ( !email_exists( $user_email_sanitized ) ) {
				return new WP_Error(
					"app_expert_invalid_email",
					__( "There is no account with that email address.", 'app-expert' ),
					array(
						'status' => 400,
					)
				);
			}

			return $user_email_sanitized;
		}


	}


}

$app_expert_forgot_password_controller = new App_Expert_Forgot_Password_Controller();
$app_expert_forgot_password_controller->register_routes();