<?php
if (!defined('ABSPATH')) {
    exit();
}

if (!class_exists('App_Expert_Notification_Controller')) {

    class App_Expert_Notification_Controller extends WP_REST_Controller
    {

        private $_runner;
        public function __construct()
        {
            $this->rest_base = 'notification';
            $this->namespace = APP_EXPERT_API_NAMESPACE;
            $this->_runner=App_Expert_DB_Handler_Handler::get_instance();
            add_filter('appexpert_authenticate_api_url',[$this,'_authenticate_api_url']);
        }
        public function register_routes()
        {
            register_rest_route($this->namespace, '/' . $this->rest_base,
                array(
                    array(
                        'methods'             => WP_REST_Server::READABLE,
                        'callback'            => array($this, 'get_items'),
                        'args'                => $this->get_notification_parameters(),
                        'permission_callback' => '__return_true',
                    ),
                )
            );
            register_rest_route($this->namespace, '/' . $this->rest_base.'/mark_as_seen' ,
                array(
                    array(
                        'methods'             => WP_REST_Server::CREATABLE,
                        'callback'            => array($this, 'mark_as_seen'),
                        'args'                => $this->get_mark_as_seen_notification_parameters(),
                        'permission_callback' => '__return_true',
                    ),
                )
            );
        }
        public function get_object_data($notificationObj){
            $local  = App_Expert_Language_Handler::get_user_locale(get_current_user_id());
            $title=json_decode($notificationObj->title);
            $notificationObj->title=$title->$local;
            $content=json_decode($notificationObj->content);
            $notificationObj->content=$content->$local;
            $notificationObj->segment=null;
            $notificationObj->attachment=null;
            if(!empty($notificationObj->attachment_id)){
                $notificationObj->attachment=wp_get_attachment_url($notificationObj->attachment_id);
            }
            return $notificationObj;
        }


        public function get_items($request)
        {
            global $wpdb;
            $response=new WP_REST_Response();
            $user = wp_get_current_user();

            $limit = $request->get_param('per_page')??5;
            $page  = $request->has_param('page')?$request->get_param('page')-1:0;
            $skip = $page * $limit;
            require_once APP_EXPERT_PATH . 'includes/handler/app-expert-notification-handler.php';
            $_notification_saver_handler = new App_Expert_Notification_Handler();

            $qCond=$_notification_saver_handler->get_condition($user);
            $q="select notification.*, IFNULL(notification_user.is_read, 0) 'is_read'
               $qCond
               order by id desc
               LIMIT {$limit} OFFSET {$skip}";
            $qCount="select count(notification.id) $qCond ";
           $data=$wpdb->get_results($q);
            $items=[];
            foreach ($data as $obj){
                $items[]=$this->get_object_data($obj);
            }
            $response->set_data($items);
            $res= App_Expert_Response::get_api_list_success_response(
                $response,
                'app_expert_notification_retrieved',
                'retrieved all notification'
            );

            $res['data']['unseenCount'] = $_notification_saver_handler->get_unseen_count($user);
            $res['data']['total_count'] = (int)$wpdb->get_var($qCount);
            $res['data']['total_pages'] = ceil((int)$res['data']['total_count']/(int)$limit);
            $res['data']['has_next'] = $res['data']['current_page']<$res['data']['total_pages'];
            $res['data']['has_prev'] = $res['data']['current_page']>1;
            return $res;
        }
        public function mark_as_seen(WP_REST_Request $request){
            $user_id = get_current_user_id();
            $notification_id=$request->get_param('notification_id');
            global $wpdb;
            $id=$wpdb->get_var("select id 
                                  from {$this->_runner->AE_User_Notification_Table_name}
                                  where user_id = '{$user_id}'
                                  and notification_id = '{$notification_id}'");
            if(!$id){
                $wpdb->insert($this->_runner->AE_User_Notification_Table_name,[
                    "user_id" => $user_id,
                    "notification_id" => $notification_id,
                    "is_read" => "1"
                ]);
            }else{
                $wpdb->update($this->_runner->AE_User_Notification_Table_name,[
                    "user_id" => $user_id,
                    "notification_id" => $notification_id,
                    "is_read" => "1"
                ],[
                    "user_id" => $user_id,
                    "notification_id" => $notification_id
                ]);
            }
            $response=new WP_REST_Response();
            $response->data=null;
            return App_Expert_Response::get_api_custom_success_response(
                [],
                'app_expert_notification_updated',
                'Notification updated'
            );
        }
        public function _authenticate_api_url($authenticate_url){
            $req = $_SERVER['REQUEST_URI'];

            if($_SERVER['REQUEST_METHOD'] !== 'OPTIONS'&&strpos($req,"/wp-json/app-expert/v1/notification")>-1 ) return true ;
            return $authenticate_url;
        }
        public function get_notification_parameters(){
            return  array(
                'page'        => array(
                    'description' => __( 'page' , 'app-expert' ),
                    'type'        => 'int',
                    'context'     => array( 'edit' ),
                    'required' => false
                ),
                'per_page'        => array(
                    'description' => __( 'count of items per page' , 'app-expert' ),
                    'type'        => 'int',
                    'context'     => array( 'edit' ),
                    'required' => false
                )
            );
        }
        public function get_mark_as_seen_notification_parameters()
        {
            return  array(
                'notification_id'        => array(
                    'description' => __( 'notification to mark as seen' , 'app-expert' ),
                    'type'        => 'int',
                    'context'     => array( 'edit' ),
                    'required' => false
                )
            );
        }

    }
}

$App_Expert_Settings_Controller = new App_Expert_Notification_Controller();
$App_Expert_Settings_Controller->register_routes();
