<?php
if (!defined('ABSPATH')) {
	exit();
}

if (!class_exists('App_Expert_Woocommerce_Checkout')) {

	class App_Expert_Woocommerce_Checkout
	{

		const MOBILE_CHECKOUT_PAGE_SLUG = 'mobile-app-checkout-page';
		const MOBILE_THANK_YOU_PAGE_SLUG = 'mobile-app-woocommerce-thank-you';


		public function __construct()
		{

			// Render custom checkout page
            add_filter('template_include', array($this, 'render_checkout_template'), 12, 1);
            add_action('template_include', array($this, 'render_thank_you_page'), 12, 1);

            add_action('template_redirect', array($this, 'add_products_to_cart'), 10, 1);
            add_action('woocommerce_thankyou', array($this, 'redirect_thank_you_page'), 10, 1);

            //fix lang in checkout
            add_action('init', array($this, 'force_switch_lang'));
            add_filter('locale', array($this, 'force_set_lang'));


			// Delete all woocommerce styles from custom checkout page
			add_action('wp_enqueue_scripts',  array($this, 'delete_theme_styles'), 20);
			add_action('wp_enqueue_scripts',  array($this, 'add_custom_pages_styles'), 20);
		}



		public function redirect_thank_you_page($order_id)
		{


			
			$isWebview = false;
			if ((strpos($_SERVER['HTTP_USER_AGENT'], 'Mobile/') !== false) && (strpos($_SERVER['HTTP_USER_AGENT'], 'Safari/') == false)) {
				$isWebview = true;
			}

			if (isset($_SERVER['HTTP_X_REQUESTED_WITH'])) {
				$isWebview = true;
			}

			if ($isWebview) {
				$thankyou_page = @get_page_by_path(self::MOBILE_THANK_YOU_PAGE_SLUG);
                $url = "?order_id=$order_id";
                if(isset($_COOKIE["ae_locale"])){
                    $url.="&ae_display_lang={$_COOKIE["ae_locale"]}";
                }
                wp_redirect(get_permalink($thankyou_page->ID) .$url );
				exit;
			}
		}


		public function render_thank_you_page($template){
			if ($this->is_thank_you_custom_page()) {
				return APP_EXPERT_PATH . "templates/woocommerce-thankyou.php";
			}

			return $template;
		}

		public function render_checkout_template($template)
		{
			if ($this->is_checkout_custom_page()) {
				return APP_EXPERT_PATH . "templates/woocommerce-cleanpage.php";
			}
			return $template;
		}


		function delete_theme_styles()
		{
			if (!$this->is_checkout_custom_page() && !$this->is_thank_you_custom_page() ) {
				return;
			}



			global $wp_styles;
			global $wp_scripts;

			$wp_get_theme = wp_get_theme();

			$child_theme  = $wp_get_theme->get_stylesheet();
			$parent_theme = $wp_get_theme->get_template();

			foreach ($wp_styles->registered as $key => $value) {
				$src = $value->src;

				// Woocommerce special theme handling
				if (strpos($src, "woocommerce/assets/css/" . $parent_theme) !== false) {
					unset($wp_styles->registered[$key]);
				}

				if (strpos($src, "woocommerce/assets/css/" . $child_theme) !== false) {
					unset($wp_styles->registered[$key]);
				}

				// Twenty themes
				if (strpos($src, "woocommerce/assets/css/twenty") !== false) {
					unset($wp_styles->registered[$key]);
				}

				// Themes assets
				if (strpos($src, "themes/") !== false) {
					unset($wp_styles->registered[$key]);
				}

				if (strpos($src, "themes/$child_theme/") !== false || strpos($src, "themes/$parent_theme/") !== false) {
					unset($wp_styles->registered[$key]);
				}

				if (strpos($src, "/uploads/$child_theme/") !== false || strpos($src, "/uploads/$parent_theme/") !== false) {
					unset($wp_styles->registered[$key]);
				}
			}

			foreach ($wp_scripts->registered as $key => $value) {
				$src = $value->src;

				if (strpos($src, "themes/") !== false) {
					unset($wp_styles->registered[$key]);
				}

				if (strpos($src, "themes/$child_theme/") !== false || strpos($src, "themes/$parent_theme/") !== false) {
					unset($wp_scripts->registered[$key]);
				}

				if (strpos($src, "/uploads/$child_theme/") !== false || strpos($src, "/uploads/$parent_theme/") !== false) {
					unset($wp_scripts->registered[$key]);
				}
			}
		}


		/**
		 * Enqueue style pages for woocommerce
		 */
		function add_custom_pages_styles()
		{
			if ($this->is_checkout_custom_page()){
				wp_enqueue_style('app-expert-custom-checkout-style', APP_EXPERT_URL . 'assets/css/woocommerce.css', false, APP_EXPERT_PLUGIN_VERSION, 'all');
				
			}elseif($this->is_thank_you_custom_page() ) {
				wp_enqueue_style('app-expert-custom-checkout-style', APP_EXPERT_URL . 'assets/css/woocommerce-thankyou.css', false, APP_EXPERT_PLUGIN_VERSION, 'all');
			}			

		}




        public static function is_checkout_custom_page()
		{
            $saved_page=get_page_by_path(self::MOBILE_CHECKOUT_PAGE_SLUG);
            if(!($saved_page instanceof WP_Post)) return false;

            global $post;
            if(!($post instanceof WP_Post)) return false;
            $languages =App_Expert_Language_Handler::get_active_languages();
            $flag=false;
            if ( !empty( $languages ) ) {
                foreach( $languages as $l ) {
                    $flag=($post->ID==apply_filters( 'wpml_object_id', $saved_page->ID, 'page', false, $l['language_code']));
                    if($flag) break;
                }
            }
			return $flag;
		}

		protected function is_thank_you_custom_page()
		{
			return is_page(self::MOBILE_THANK_YOU_PAGE_SLUG);
		}


		function add_products_to_cart()
		{
			if (!function_exists('WC')) {
				return;
			}

			if (!$this->is_checkout_custom_page()) {
				return;
			}

			WC()->cart->empty_cart();

			// Product details has the format:
			// product1_id,variation1_id,quantity1,variation_name:variation_value;;variation_name:variation_value||product2_id,...

			$products = $_GET['products'] ? explode("||", urldecode($_GET['products'])) : [];
			foreach ($products as $product) {
				$product_details = explode(",", $product);
				$product_id = intval($product_details[0]);
				$variation_id = intval($product_details[1]);
				$quantity = intval($product_details[2]);
				$variations = isset($product_details[3]) ? explode(";;", $product_details[3]) : [];
				$variationsData = [];

				// Get variation data key value
				foreach ($variations as $sent_data) {
					$variation = explode(":", $sent_data);
					$variationsData[$variation[0]] = $variation[1];
				}

				WC()->cart->add_to_cart($product_id, $quantity, $variation_id, $variationsData);
			}
		}

        public function force_set_lang($locale){
            if(isset($_GET['ae_display_lang']))
            {
                return $_GET['ae_display_lang'];
            }
            else if(isset($_COOKIE["ae_locale"]))
            {
                return $_COOKIE["ae_locale"];
            }

            return $locale;
        }
        public function force_switch_lang(){
            if(isset($_GET['ae_display_lang']))
            {
                App_Expert_Language_Handler::switch_lang($_GET['ae_display_lang']);
            }
            else if(isset($_COOKIE["ae_locale"]))
            {
                App_Expert_Language_Handler::switch_lang($_COOKIE["ae_locale"]);
            }
        }
	}
}
new App_Expert_Woocommerce_Checkout();
