<?php
if (!defined('ABSPATH')) {
    exit();
}

if (!class_exists('App_Expert_WC_Orders_Controller')) {
       
    /**
     * Add Woocommerce write api for orders
     */
    class App_Expert_WC_Orders_Controller {
        public function __construct() {
            $this->rest_base = '/orders';
            $this->namespace = APP_EXPERT_API_NAMESPACE;
        }
        
        public function register_routes() {
            register_rest_route(
                $this->namespace,
                '/' . $this->rest_base . '/cancel/(?P<id>[\d]+)',
                array(
                    'args'   => array(
                        'id' => array(
                            'description' => __( 'Unique identifier for the order.' ),
                            'type'        => 'integer',
                        ),
                    ),
                    array(
                        'methods'  => WP_REST_Server::CREATABLE,
                        'callback' => array( $this, 'cancel_order' ),
                        'permission_callback' => '__return_true',
                    ),
                )
            );
        }
        
        public function cancel_order($request) {
            $order_id = $request['id'];
            $user_id = App_Expert_Request::get_user_id($request);//$request['user_id'];
            $order = wc_get_order( $order_id );
            $order_can_cancel = $order->has_status( apply_filters( 'woocommerce_valid_order_statuses_for_cancel', array( 'pending', 'failed' ), $order ) );

            // check if order with this id exists
            if($order){
                // check if user can cancel order
                if($order->get_customer_id() == $user_id){
                    // check if order can be cancelled
                    if($order_can_cancel){
                        $order->update_status( 'cancelled', __( 'Order cancelled by customer.', 'woocommerce' ) );
                        return App_Expert_Response::get_api_custom_success_response(array('status' => 200), 'app_expert_success', __('Order was cancelled successfully', 'app-expert'));
                    }else{
                        return new WP_Error(
                            'app_expert_not_acceptable',
                            __('Order has invalid status for cancel', 'app-expert'),
                            array(
                                'status' => 406
                            )
                        );
                    }
                }else{
                    return new WP_Error(
                        'app_expert_forbidden_action',
                        __('You are not authorized to cancel this order', 'app-expert'),
                        array(
                            'status' => 403
                        )
                    );
                }
            }else{
                return new WP_Error(
                    'app_expert_not_found',
                    __('No order found', 'app-expert'),
                    array(
                        'status' => 404
                    )
                );
            }
        }
    }

    $app_expert_orders_controller = new App_Expert_WC_Orders_Controller();
    $app_expert_orders_controller->register_routes();
}
