<?php
if (!defined('ABSPATH')) {
    exit();
}

if (!class_exists('App_Expert_WC_Rest_Products_Controller')) {

    if (class_exists('WC_REST_Products_Controller')) {
       
        /**
         * Extends Woocommerce products endpoint to allow filter by multiple attributes
         * @inheritDoc
         */
        class App_Expert_WC_Rest_Products_Controller extends WC_REST_Products_Controller
        {
            public function __construct()
            {
                require_once APP_EXPERT_PATH . 'includes/handler/app-expert-wc-product-handler.php';
                add_filter('woocommerce_rest_prepare_product_object', array($this, 'add_products_edits'), 20, 3);
                add_filter('woocommerce_rest_product_object_query',array($this,'add_to_query'),10,2);
                add_filter( "rest_product_collection_params", array($this,'add_additional_params_to_product_api'), 10, 1 );
            }

            public function add_to_query($args, $request)
            {
                if (!empty($request['appexpert_attributes']) && is_array($request['appexpert_attributes'])) {

                    $additional_tax_query = [];
                    $all_available_attr_names = wc_get_attribute_taxonomy_names();
                    foreach ($request['appexpert_attributes'] as $attribute_name => $attribute_term_ids) {
                        if (in_array($attribute_name, $all_available_attr_names, true)) {
                            $additional_tax_query[] = array(
                                'taxonomy' => $attribute_name,
                                'field'    => 'term_id',
                                'terms'    => explode(",", $attribute_term_ids),
                            );
                        }
                    }
                    if (!isset($args['tax_query'])) {
                        $args['tax_query'] = [];
                    }
                    $args['tax_query'] = array_merge($additional_tax_query, $args['tax_query']); // WPCS: slow query ok.
                }

                if(isset($_GET['orderby'])){
                    if ('rating' == $_GET['orderby']) {
                        if(isset($_GET['order'])){
                            $args['orderby'] = 'meta_value_num';
                            $args['order'] = $_GET['order'];
                            $args['meta_key'] = '_wc_average_rating';
                        }else{
                            $args['orderby'] = 'meta_value_num';
                            $args['order'] = 'DESC';
                            $args['meta_key'] = '_wc_average_rating';
                        }
                    }
                }
                
                return $args;
            }

            public function add_products_edits($response, $product)
            {
                if (empty($response->data))
                    return $response;

                $response->data= App_Expert_WC_Product_Handler::get_data($response->data,$product);
                return $response;
            }
            public function fetch_variation_params()
            {
                return [
                    "attributes"=>  array(
                        'description'       => __( 'attributes to select specific variation', 'woocommerce' ),
                        'type'              => 'Array',
                        'required'          => true,
                        'items'       => array(
                            'type' => 'Array',
                            'items'       => array(
                                'id' => 'integer',
                                'name' => 'string',
                                'value' => 'string',
                            ),
                        ),
                    ),
                ];
            }
            public function register_routes()
            {
                register_rest_route(
                    $this->namespace,
                    '/' . $this->rest_base."/(?P<product_id>[\d]+)/fetch-variation",
                    array(
                        'args'   => array(
                            'product_id' => array(
                                'description' => __( 'Unique identifier for the variable product.', 'woocommerce' ),
                                'type'        => 'integer',
                            ),
                        ),
                        array(
                            'methods'             => WP_REST_Server::CREATABLE,
                            'callback'            => array( $this, 'fetch_variation' ),
                            'permission_callback' => array( $this, 'get_items_permissions_check' ),
                            'args'                => $this->fetch_variation_params(),
                        ),
                        'schema' => array( $this, 'get_public_item_schema' ),
                    )
                );
            }
            public function fetch_variation(WP_REST_Request $request){
                $product_id=$request->get_param('product_id');
                $prod = wc_get_product($product_id);
                $attrs=$request->get_param('attributes');
                $attributes=[];
                foreach ($attrs as $attr){
                    $attr_name=strtolower($attr['name']);
                    $prefix="attribute_";
                    if($attr["id"]){
                        $attr_name="pa_".strtolower($attr['name']);
                        $term=get_term_by('name',$attr['value'],$attr_name);
                        $attr['value']=$term->slug;
                    }
                    $attributes[$prefix.$attr_name]=$attr['value'];
                }

                $_store=new \WC_Product_Data_Store_CPT();

                $variation_id= $_store->find_matching_product_variation($prod, $attributes);
                if($variation_id>0){
                    $new_req=new WP_REST_Request();
                    $new_req->set_param('id',$variation_id);
                    return $this->get_item($new_req);
                }
                return new WP_Error("invalid_selection",__("No product found with the selected values",'woocommerce'),["status"=>422]);

            }

            function add_additional_params_to_product_api ($params){
                $params['appexpert_attributes'] = array(
                    'description' => __( '(Array) filter with multiple attribute. example:appexpert_attributes[pa_color]=20,30' ),
                    'type'        => 'Array',
                );
                return $params;
            }
        }

        $app_expert_wc_products_controller = new App_Expert_WC_Rest_Products_Controller();
        $app_expert_wc_products_controller->register_routes();
    }
}
